package registry_test

import (
	"flow/internal/assert"
	"flow/registry"
	"strings"
	"testing"
)

func TestMakeBatch(t *testing.T) {
	a := assert.A(t)
	r := registry.New()

	b := registry.Describer(
		r.MustAdd(strings.Split, strings.Join),
		r.MustAdd(strings.Compare),
		r.MustRegister("named", strings.Compare),
	)
	a.Eq(len(b), 4, "should have 3 entries in batch")

	b.Inputs("str")
	b.Output("result")

	for _, en := range b {
		a.Eq(en.Description.Inputs[0].Name, "str", "first input should be string")
		a.Eq(en.Description.Output.Name, "result", "output should be equal")
	}
}
func TestNilDescription(t *testing.T) {
	a := assert.A(t)
	r := registry.New()

	r.MustRegister("test", strings.Compare)
	e, _ := r.Entry("test")
	e.Description = nil

	b := registry.Describer(e)
	a.Eq(len(b), 1, "should have 3 entries in batch")

	// Testing adding to a nil description
	b.Tags("")
	b.Description("")
	b.Inputs("str")
	b.Output("result")
	b.Extra("v", "v")

}
