package flowbuilder

// Node that will contain registry src
import "log"

type Node struct {
	ID            string            `json:"id"`
	Src           string            `json:"src"`
	Label         string            `json:"label"`
	DefaultInputs map[int]string    `json:"defaultInputs"`
	Prop          map[string]string `json:"prop"`
}

// Link that joins two nodes
type Link struct {
	From string `json:"from"`
	To   string `json:"to"`
	In   int    `json:"in"`
}

// Trigger that join two nodes on state change
type Trigger struct {
	From string   `json:"from"`
	To   string   `json:"to"`
	On   []string `json:"on"`
}

// FlowDocument flow document
type FlowDocument struct {
	Nodes    []Node    `json:"nodes"`
	Links    []Link    `json:"links"`
	Triggers []Trigger `json:"triggers"`
}

func (fd *FlowDocument) fetchNodeByID(ID string) *Node {
	for _, n := range fd.Nodes {
		if n.ID == ID {
			return &n
		}
	}
	return nil
}

func (fd *FlowDocument) fetchTriggerFrom(ID string) []Trigger {
	ret := []Trigger{}
	for _, t := range fd.Triggers {
		if t.From != ID {
			continue
		}
		log.Println("Trigger is:", ID, "adding")
		ret = append(ret, t)
	}
	return ret
}

func (fd *FlowDocument) fetchLinksTo(ID string) []Link {
	ret := []Link{}
	for _, l := range fd.Links {
		if l.To != ID {
			continue
		}
		ret = append(ret, l)
	}
	return ret
}
func (fd *FlowDocument) fetchLinkTo(ID string, n int) *Link {
	for _, l := range fd.Links {
		if l.To != ID || l.In != n {
			continue
		}
		return &l
	}
	return nil
}
