package flowserver

import (
	"flow/registry"
	"log"
	"net/http"
	"net/http/httputil"
	"net/url"
	"os"
	"strings"

	"github.com/gohxs/webu"
)

//go:generate go get github.com/gohxs/genversion
//go:generate genversion -package flowserver -out version.go
//

// FlowServer structure
type FlowServer struct {
	mux *http.ServeMux
}

// New creates a New flow server
func New(r *registry.R, store string) *FlowServer {
	if r == nil {
		r = registry.Global.Clone()
	}
	mux := http.NewServeMux()
	mux.Handle("/conn", NewFlowSessionManager(r, store))

	if os.Getenv("DEBUG") == "1" {
		log.Println("DEBUG MODE: reverse proxy localhost:8081")
		proxyURL, err := url.Parse("http://localhost:8081")
		if err != nil {
			return nil
		}
		mux.Handle("/", httputil.NewSingleHostReverseProxy(proxyURL))
	} else {
		mux.Handle("/", webu.StaticHandler("web", "index.html"))
	}

	return &FlowServer{mux}
}

func (f *FlowServer) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	// Manual routing here
	// Grab last part of path
	urlParts := strings.Split(r.URL.Path, "/")
	prefixToRemove := strings.Join(urlParts[:len(urlParts)-1], "/")
	http.StripPrefix(prefixToRemove, f.mux).ServeHTTP(w, r)

}

// ListenAndServe starts the httpserver
// It will listen on default port 2015 and increase if port is in use
func (f *FlowServer) ListenAndServe(addr string) error {
	return http.ListenAndServe(addr, f.mux)
}
